class DataContract < Module
  class Contract
    attr_reader :mod
    attr_reader :obj

    def initialize(mod, obj)
      @mod = mod
      @obj = obj
    end

    def setters
      mod.instance_methods(false).select { |m| m.to_s[-1, 1] == '=' }
    end

    def getters
      mod.instance_methods(false).select { |m| m.to_s[-1, 1] != '=' }
    end

    def compatible?(other_obj)
      mod.instance_methods(false).each do |m|
        return false unless other_obj.respond_to?(m, true)
      end
      return true
    end

    def shared?(other_obj)
      other_obj.is_a? mod
    end

    def assignable?(other_obj)
      unless shared? other_obj
        unless compatible? other_obj
          return false
        end
      end
      true
    end

    def scatter(receiver)
      getters.each do |attribute|
        val = obj.send attribute
        assign val, receiver, attribute
      end
      receiver
    end

    def assign(val, receiver, attribute)
      setter = "#{attribute}="

      unless receiver.respond_to?(setter, true)
        raise ContractError, "The #{attribute} attribute cannot be assigned to #{receiver.class.name}"
      end

      receiver.send setter, val
    end
  end
end
